import { AutoMap } from '@automapper/classes';
import {
  IsOptional,
  IsString,
  IsBoolean,
  IsDate,
  IsInt,
  IsEnum,
  ValidateNested,
  ArrayMinSize,
  IsUUID,
} from 'class-validator';
import { Address } from '../entities/address';
import { Frequency } from '../types/frequency.enum';
export class Ad {
  @IsUUID(4)
  @AutoMap()
  uuid: string;

  @IsUUID(4)
  @AutoMap()
  userUuid: string;

  @IsBoolean()
  @AutoMap()
  driver: boolean;

  @IsBoolean()
  @AutoMap()
  passenger: boolean;

  @IsEnum(Frequency)
  @AutoMap()
  frequency: Frequency;

  @IsDate()
  @AutoMap()
  fromDate: Date;

  @IsDate()
  @AutoMap()
  toDate: Date;

  @IsOptional()
  @IsDate()
  @AutoMap()
  monTime?: string;

  @IsOptional()
  @IsString()
  @AutoMap()
  tueTime?: string;

  @IsOptional()
  @IsString()
  @AutoMap()
  wedTime?: string;

  @IsOptional()
  @IsString()
  @AutoMap()
  thuTime?: string;

  @IsOptional()
  @IsString()
  @AutoMap()
  friTime?: string;

  @IsOptional()
  @IsString()
  @AutoMap()
  satTime?: string;

  @IsOptional()
  @IsString()
  @AutoMap()
  sunTime?: string;

  @IsInt()
  @AutoMap()
  monMargin: number;

  @IsInt()
  @AutoMap()
  tueMargin: number;

  @IsInt()
  @AutoMap()
  wedMargin: number;

  @IsInt()
  @AutoMap()
  thuMargin: number;

  @IsInt()
  @AutoMap()
  friMargin: number;

  @IsInt()
  @AutoMap()
  satMargin: number;

  @IsInt()
  @AutoMap()
  sunMargin: number;

  @IsInt()
  @AutoMap()
  seatsDriver: number;

  @IsInt()
  @AutoMap()
  seatsPassenger: number;

  @IsBoolean()
  @AutoMap()
  strict: boolean;

  @IsDate()
  @AutoMap()
  createdAt: Date;

  @IsDate()
  @AutoMap()
  updatedAt?: Date;

  @ArrayMinSize(2)
  @ValidateNested({ each: true })
  @AutoMap(() => [Address])
  addresses: Address[];
}
