import { NotFoundException } from '@nestjs/common';
import { Test, TestingModule } from '@nestjs/testing';
import { Messager } from '../../adapters/secondaries/messager';
import { FindAdByUuidQuery } from '../../queries/find-ad-by-uuid.query';
import { AdsRepository } from '../../adapters/secondaries/ads.repository';
import { FindAdByUuidUseCase } from '../../domain/usecases/find-ad-by-uuid.usecase';
import { FindAdByUuidRequest } from '../../domain/dtos/find-ad-by-uuid.request';

const mockAd = {
  uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
};

const mockAdRepository = {
  findOneByUuid: jest
    .fn()
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    .mockImplementationOnce((query?: FindAdByUuidQuery) => {
      return Promise.resolve(mockAd);
    })
    .mockImplementation(() => {
      return Promise.resolve(null);
    }),
};

const mockMessager = {
  publish: jest.fn().mockImplementation(),
};

describe('FindAdByUuidUseCase', () => {
  let findAdByUuidUseCase: FindAdByUuidUseCase;
  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [],
      providers: [
        {
          provide: AdsRepository,
          useValue: mockAdRepository,
        },
        {
          provide: Messager,
          useValue: mockMessager,
        },
        FindAdByUuidUseCase,
      ],
    }).compile();

    findAdByUuidUseCase = module.get<FindAdByUuidUseCase>(FindAdByUuidUseCase);
  });
  it('should be defined', () => {
    expect(findAdByUuidUseCase).toBeDefined();
  });
  describe('execute', () => {
    it('should return an ad', async () => {
      const findAdByUuidRequest: FindAdByUuidRequest =
        new FindAdByUuidRequest();
      findAdByUuidRequest.uuid = 'bb281075-1b98-4456-89d6-c643d3044a91';
      const user = await findAdByUuidUseCase.execute(
        new FindAdByUuidQuery(findAdByUuidRequest),
      );
      expect(user).toBe(mockAd);
    });
    it('should throw an error if ad does not exist', async () => {
      const findAdByUuidRequest: FindAdByUuidRequest =
        new FindAdByUuidRequest();
      findAdByUuidRequest.uuid = 'bb281075-1b98-4456-89d6-c643d3044a90';
      await expect(
        findAdByUuidUseCase.execute(new FindAdByUuidQuery(findAdByUuidRequest)),
      ).rejects.toBeInstanceOf(NotFoundException);
    });
  });
});
